/*
 * MyEP_STM32F103_QEI.c
 	 	QEI1		QEI2
 TIMER	2			4
 A 		PORTA_15	PORTB_6
 B 		PORTB_3		PORTB_7
 */
 
 /*
 De momento est QEI1 en PORTA0 y PORTA1, que no son tolerantes a 5V
 Utilizar QEI2 (PORTB6 y 7 son tolerantes a 5V)
 
 */

#include <stdint.h>
#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_tim.h"
#include "MyEP_STM32F103_QEI.h"
#include "misc.h"


extern volatile int32_t QEI1Cont;
extern volatile int32_t QEI2Cont;

/*  Lectura de posicin */
int32_t QEI_Read (uint8_t canal)
{
  if (canal==1) return TIM_GetCounter (QEI1_TIMER)+QEI1Cont;
  if (canal==2) return TIM_GetCounter (QEI2_TIMER)+QEI2Cont;
  return 0;
}

/* Inicializacin de posicin */
void QEI_Reset (uint8_t canal)
{
  __disable_irq();
  switch(canal)
  {
  case 1:
	  TIM_SetCounter (QEI1_TIMER, 0);
	  break;
  case 2:
	  TIM_SetCounter (QEI2_TIMER, 0);
	  break;
  default:
	  break;
  }
  __enable_irq();
}

void QEI_Init(uint8_t canal)
{
	/* Mapeo de pines de QEI1 y QEI2 */
	// -------------QEI1
GPIO_InitTypeDef GPIO_InitStructure;				// Estructuras paa inicializar pines
NVIC_InitTypeDef NVIC_InitStructure;  			// e interrupciones del QEI
switch(canal)
	{
	case 1:
	  GPIO_PinRemapConfig( GPIO_PartialRemap1_TIM2, ENABLE );// PA15 Y PB3 tolerantes a 5 VOLTS RM0008 pp 156 Tabla 41

	  RCC_APB2PeriphClockCmd (QEI1_A_GPIO_CLK, ENABLE);
	  RCC_APB2PeriphClockCmd (QEI1_B_GPIO_CLK, ENABLE);
	  GPIO_StructInit (&GPIO_InitStructure);
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	  GPIO_InitStructure.GPIO_Pin = QEI1_A_PIN;
	  GPIO_Init (QEI1_A_GPIO_PORT, &GPIO_InitStructure);
	  GPIO_InitStructure.GPIO_Pin = QEI1_B_PIN;
	  GPIO_Init (QEI1_B_GPIO_PORT, &GPIO_InitStructure);


	  RCC_APB1PeriphClockCmd (QEI1_TIMER_CLK, ENABLE);
	  TIM_EncoderInterfaceConfig (QEI1_TIMER, TIM_EncoderMode_TI12,
	                              TIM_ICPolarity_Rising,
	                              TIM_ICPolarity_Rising);
	  TIM_SetAutoreload (QEI1_TIMER, 0xffff);
	  TIM_UpdateRequestConfig(QEI1_TIMER, TIM_UpdateSource_Regular);
	  TIM_ITConfig(QEI1_TIMER, TIM_IT_Update, ENABLE);
	  NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
      NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;


	  NVIC_Init(&NVIC_InitStructure);
	  TIM_Cmd (QEI1_TIMER, ENABLE);
	  QEI_Reset(canal);
	  break;
	case 2:
	  RCC_APB2PeriphClockCmd (QEI2_A_GPIO_CLK, ENABLE);
	  RCC_APB2PeriphClockCmd (QEI2_B_GPIO_CLK, ENABLE);
	  GPIO_StructInit (&GPIO_InitStructure);
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	  GPIO_InitStructure.GPIO_Pin = QEI2_A_PIN;
	  GPIO_Init (QEI2_A_GPIO_PORT, &GPIO_InitStructure);
	  GPIO_InitStructure.GPIO_Pin = QEI2_B_PIN;
	  GPIO_Init (QEI2_B_GPIO_PORT, &GPIO_InitStructure);
	  RCC_APB1PeriphClockCmd (QEI2_TIMER_CLK, ENABLE);
	  TIM_EncoderInterfaceConfig (QEI2_TIMER, TIM_EncoderMode_TI12,
	                              TIM_ICPolarity_Rising,
	                              TIM_ICPolarity_Rising);
	  TIM_SetAutoreload (QEI2_TIMER, 0xffff);
	  TIM_UpdateRequestConfig(QEI2_TIMER, TIM_UpdateSource_Regular);
	  TIM_ITConfig(QEI2_TIMER, TIM_IT_Update, ENABLE);
	  NVIC_InitStructure.NVIC_IRQChannel = TIM4_IRQn;
      NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;  // Prioridad de interru
	  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	  NVIC_Init(&NVIC_InitStructure);
	  TIM_Cmd (QEI2_TIMER, ENABLE);
	  QEI_Reset(canal);
	  break;
	default:
		break;
	}



	  // Connect the pins to their Alternate Functions
/*	  GPIO_PinAFConfig (QEI1_A_GPIO_PORT, QEI1_A_SOURCE, QEI1_A_AF);
	  GPIO_PinAFConfig (QEI1_B_GPIO_PORT, QEI1_B_SOURCE, QEI1_B_AF);
	  GPIO_PinAFConfig (QEI2_A_GPIO_PORT, QEI2_A_SOURCE, QEI2_A_AF);
	  GPIO_PinAFConfig (QEI2_B_GPIO_PORT, QEI2_B_SOURCE, QEI2_B_AF);
*/
	  // Timer peripheral clock enable

	  // set them up as encoder inputs
	  // set both inputs to rising polarity to let it use both edges




	  // turn on the timer/counters
}
