/*******************************************************************************
* Ej03_STM32F103_Timer_UART_PWM
* Implementa Salidas PWM y Entradas de Encoder en cuadratura (QEI)
* Timers como interfaces para encoders incrementales de seal
* en cuadratura (QEI)
*
* UART1 Tx: PA9. Conectar a Rx de puente serie-usb
* UART1 Rx: PA10. Conectar a Tx de puente serie-usb.

* QEI1: TIMER2, seal A en pin PA15, B en pin PB3  (de momento en PA0, PA1)
* QEI2: TIMER4, seal A en pin PB6, B en pin PB7  (tolerantes a 5 Volts, usar este)
*
* PWM1: TIMER3, canal 1 en PA6
* PWM2: TIMER3, canal 2 en PA7
* PWM3: TIMER3, canal 3 en PB0   --+--
* PWM4: TIMER3, canal 4 en PB1   --+-- Utilizamos estos para el L298
*
* Uso:
* Para leer conteo de encoders
* :E1 lee encoder 1
* :E2 lee encoder 2 (el que usamos en nuestro caso)
*
******************************************************************************/
/* Includes */
#include "stm32f10x.h"
#include "stm32f10x_usart.h"
#include <stdio.h>
#include <stdlib.h> //para atoi y atol
#include <stdint.h>
// propias
#include "MyEP_STM32F103_GPIO.h"
#include "MyEP_STM32F103_UART.h"
#include "MyEP_STM32F103_TIMERS.h"
#include "MyEP_STM32F103_PWM.h"
#include "MyEP_STM32F103_QEI.h"

#define MAX_STRLEN 40
char USART_TxStream[MAX_STRLEN]; //buffer para sprintf
char comando[MAX_STRLEN]; // buffer de recepcin para UART
uint8_t flagRx=0; uint8_t indRx=0;

volatile uint32_t Tiempo;

volatile int32_t QEI1Cont=0;
volatile int32_t QEI2Cont=0;

volatile uint16_t PWMPrescaler;
volatile uint16_t PWMPeriodo;
volatile uint16_t PWMDuty50;
volatile uint16_t PWMDuty[5];

volatile uint32_t Ciclos=0;

#define UARTN 1

int main(void)
{
  PWMPeriodo=9000;
  PWMDuty50=PWMPeriodo/2;
  SystemInit(); // Configura PLL etc
  SystemCoreClockUpdate();                      // Get Core Clock Frequency
	#ifndef __NO_SYSTICK
	  SysTick_Config(SystemCoreClock /1000);        // SysTick 1 msec irq
	#endif
  QEI_Init(2);  // Encoder 2, PB6 y PB7, tolerante a 5 volts

  TIMER_init(3,1,PWMPeriodo); //TIM3 para PWM: 36MHz/1(prescaler)=36000000Hz/9000(periodo) = 4kHz

  PWMDuty[1]=PWM_Init(1, PWMDuty50, 0);   // Canal 1 de Timer3 (A6), pulsos en alto, polaridad
  PWMDuty[2]=PWM_Init(2, PWMDuty50, 1);	// Canal 2 de Timer3 (A7), pulsos en alto, polaridad positiva
  PWMDuty[3]=PWM_Init(3, PWMDuty50, 0);   // Canal 3 de Timer3 (B0), pulsos en alto, polaridad
  PWMDuty[4]=PWM_Init(4, PWMDuty50, 1);	// Canal 4 de Timer3 (B1), pulsos en alto, polaridad positiva

  Pin_Init(GPIOC, GPIO_Pin_13, GPIO_Mode_Out_PP, GPIO_Speed_2MHz);

  UART_init(UARTN,250000,1,0); // Uart, 250000 bps, Interr Rx, no Interr Tx

  sprintf(USART_TxStream," UART-QEI-PWM MyEp 2019\n\r");
  USART_puts(USART1,USART_TxStream);

  while (1)
  {
	  GPIO_SetBits(GPIOC, GPIO_Pin_13 );
	  Tiempo=1000000;
	  while(Tiempo--);
	  GPIO_ResetBits(GPIOC, GPIO_Pin_13 );
	  Tiempo=1000000;
	  while(Tiempo--);
  }
}




void interpreta(void)
{
	int32_t valor32;
	uint16_t valor16;
	uint8_t canal;
	valor16=0;
	valor32=0;
	switch(comando[0])
		{
		case 't':
		case 'T':  // :Tnnn  , ajuste de periodo en pulsos
			if(comando[1]){
				valor16=atoi(&comando[1]);
				PWMPeriodo=PWM_Period(valor16);
				}
			sprintf(USART_TxStream,"Periodo PWM: %d\r\n",PWMPeriodo);
			USART_puts(USART1,USART_TxStream);
			break;
		case 'p':
		case 'P':   // :Pnnnn  ajuste de prescaler (division de clk de sistema por nnnn
			if(comando[1]){
				valor16=atol(&comando[1]);
				PWMPrescaler=PWM_Prescaler(valor16);
				}
			sprintf(USART_TxStream,"Prescaler PWM: %d\r\n",PWMPrescaler);
			USART_puts(USART1,USART_TxStream);
			break;
		case 'd':
		case 'D':  // :DX,nnn Canal X de pwm, nnn pulsos en alto (ajuste de duty cycle)
			canal=comando[1]-'0'; //transforma ASCII en nmero (Ej: '3'-'0'=51-48=3
			if(comando[3]){
				valor16=atoi(&comando[3]);
				PWMDuty[canal]=PWM_Duty(canal, valor16);
				}
			sprintf(USART_TxStream,"PWM Duty %d: %d\r\n",canal,PWMDuty[canal]);
			USART_puts(USART1,USART_TxStream);
			break;
		case 'x':
		case 'X':  // :Xnnn Ambos canales de pwm, nnn pulsos/complementarios (ajuste de duty cycle)
			if(comando[1]){
				valor16=atoi(&comando[1]);
				PWMDuty[1]=PWM_Duty(1, valor16);
				PWMDuty[2]=PWM_Duty(2, valor16);
				}
			sprintf(USART_TxStream,"PWM Duty X: %d\r\n",PWMDuty[1]);
			USART_puts(USART1,USART_TxStream);
			break;
		case 'y':
		case 'Y':  // :Ynnn Ambos canales de pwm, nnn pulsos/complementarios (ajuste de duty cycle)
			if(comando[1]){
				valor16=atoi(&comando[1]);
				PWMDuty[3]=PWM_Duty(3, valor16);
				PWMDuty[4]=PWM_Duty(4, valor16);
				}
			sprintf(USART_TxStream,"PWM Duty Y: %d\r\n",PWMDuty[3]);
			USART_puts(USART1,USART_TxStream);
			break;
		case 'c':
		case 'C':  // :C  responde con Ciclos (contador de ciclos transcurridos, interrupciones de SysTick)
			sprintf(USART_TxStream,"Ciclos:%ld\r\n",Ciclos);
			USART_puts(USART1,USART_TxStream);
			break;
		case 'e':
		case 'E':  // :EX Lee encoder X
			if(comando[1]=='1') canal=1;
			if(comando[1]=='2') canal=2;
			valor32 = QEI_Read(canal);
			sprintf(USART_TxStream,"Encoder %d,%ld\r\n",canal,valor32);
			USART_puts(USART1,USART_TxStream);
			break;
		default:
			break;
		}

}

void USART1_IRQHandler(void)
{
/*	uint8_t flagCom=0; uint8_t indcom=0;   con '='
	uint8_t flagRx=0; uint8_t indRx=0;     con ':' */
if( USART_GetITStatus(USART1, USART_IT_RXNE) )
	{
		char dato = USART1->DR;

		switch(dato)
			{
			case ':': indRx=0; flagRx=1;
					break;
			case '\r':
			case ';':
					if (flagRx)
					{
					flagRx=0;
					comando[indRx]=0;
					interpreta();
					}
					break;
			case 8:
				if (flagRx)
					{if (indRx) indRx--;}
					break;
			default:
					if (flagRx)
					{
					comando[indRx]=dato;
					if(indRx<MAX_STRLEN-1)indRx++;
					}
					break;

			}
	}
}

void TIM2_IRQHandler(void)  // Desborde de Encoder 1, incremento/decremento
                            // de QEI1Cont (ampla rango a 32 bits , ver QEI_Read())
{
	if( (TIM2->SR) & TIM_SR_UIF )	//Flag de Interrupcion por Timer Update
	  {
		  if (TIM_GetCounter (QEI1_TIMER)<0x7FFF) QEI1Cont+=(1<<16);
		  else QEI1Cont-=(1<<16);
		  TIM2->SR&=~(TIM_SR_UIF);
	  }
}
void TIM4_IRQHandler(void)  // Desborde de Encoder 2
{
	if( (TIM4->SR) & TIM_SR_UIF )	//Flag de Interrupcion por Timer Update
	  {
		  if (TIM_GetCounter (QEI2_TIMER)<0x7FFF) QEI2Cont+=(1<<16);
		  else QEI2Cont-=(1<<16);
		  TIM4->SR&=~(TIM_SR_UIF);
	  }
}


void SysTick_Handler(void) 		//Handler de SysTick
{
  Ciclos++;
}

